const express = require('express');

const Booking_samagri_mappingsService = require('../services/booking_samagri_mappings');
const Booking_samagri_mappingsDBApi = require('../db/api/booking_samagri_mappings');
const wrapAsync = require('../helpers').wrapAsync;

const router = express.Router();

/**
 *  @swagger
 *  components:
 *    schemas:
 *      Booking_samagri_mappings:
 *        type: object
 *        properties:

 *          booking_id:
 *            type: integer
 *            format: int64
 *          samagri_id:
 *            type: integer
 *            format: int64
 *          no_of_standard_qty:
 *            type: integer
 *            format: int64

 */

/**
 *  @swagger
 * tags:
 *   name: Booking_samagri_mappings
 *   description: The Booking_samagri_mappings managing API
 */

/**
 *  @swagger
 *  /api/booking_samagri_mappings:
 *    post:
 *      security:
 *        - bearerAuth: []
 *      tags: [Booking_samagri_mappings]
 *      summary: Add new item
 *      description: Add new item
 *      requestBody:
 *        required: true
 *        content:
 *          application/json:
 *            schema:
 *              properties:
 *                data:
 *                  description: Data of the updated item
 *                  type: object
 *                  $ref: "#/components/schemas/Booking_samagri_mappings"
 *      responses:
 *        200:
 *          description: The item was successfully added
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Booking_samagri_mappings"
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        405:
 *          description: Invalid input data
 *        500:
 *          description: Some server error
 */

router.post('/', async (req, res) => {

  await Booking_samagri_mappingsService.create(
    req.body.data,
    req.currentUser,
    true,
    req.headers.referer,
  );
  const payload = true;

  res.status(200).send(payload);
});

/**
 *  @swagger
 *  /api/booking_samagri_mappings/{id}:
 *    put:
 *      security:
 *        - bearerAuth: []
 *      tags: [Booking_samagri_mappings]
 *      summary: Update the data of the selected item
 *      description: Update the data of the selected item
 *      parameters:
 *        - in: path
 *          name: id
 *          description: Item ID to update
 *          required: true
 *          schema:
 *            type: string
 *      requestBody:
 *        description: Set new item data
 *        required: true
 *        content:
 *          application/json:
 *            schema:
 *              properties:
 *                id:
 *                  description: ID of the updated item
 *                  type: string
 *                data:
 *                  description: Data of the updated item
 *                  type: object
 *                  $ref: "#/components/schemas/Booking_samagri_mappings"
 *              required:
 *                - id
 *      responses:
 *        200:
 *          description: The item data was successfully updated
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Booking_samagri_mappings"
 *        400:
 *          description: Invalid ID supplied
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Item not found
 *        500:
 *          description: Some server error
 */

router.put(
  '/:id',
  wrapAsync(async (req, res) => {

    await Booking_samagri_mappingsService.update(
      req.body.data,
      req.body.id,
      req.currentUser,
    );
    const payload = true;

    res.status(200).send(payload);
  }),
);

/**
 * @swagger
 *  /api/booking_samagri_mappings/{id}:
 *    delete:
 *      security:
 *        - bearerAuth: []
 *      tags: [Booking_samagri_mappings]
 *      summary: Delete the selected item
 *      description: Delete the selected item
 *      parameters:
 *        - in: path
 *          name: id
 *          description: Item ID to delete
 *          required: true
 *          schema:
 *            type: string
 *      responses:
 *        200:
 *          description: The item was successfully deleted
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Booking_samagri_mappings"
 *        400:
 *          description: Invalid ID supplied
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Item not found
 *        500:
 *          description: Some server error
 */

router.delete(
  '/:id',
  wrapAsync(async (req, res) => {

    await Booking_samagri_mappingsService.remove(
      req.params.id,
      req.currentUser,
    );
    const payload = true;

    res.status(200).send(payload);
  }),
);

/**
 *  @swagger
 *  /api/booking_samagri_mappings:
 *    get:
 *      security:
 *        - bearerAuth: []
 *      tags: [Booking_samagri_mappings]
 *      summary: Get all booking_samagri_mappings
 *      description: Get all booking_samagri_mappings
 *      responses:
 *        200:
 *          description: Booking_samagri_mappings list successfully received
 *          content:
 *            application/json:
 *              schema:
 *                type: array
 *                items:
 *                  $ref: "#/components/schemas/Booking_samagri_mappings"
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Data not found
 *        500:
 *          description: Some server error
 */

router.get(
  '/',
  wrapAsync(async (req, res) => {

    const payload = await Booking_samagri_mappingsDBApi.findAll(req.query);

    res.status(200).send(payload);
  }),
);

router.get('/autocomplete', async (req, res) => {

  const payload = await Booking_samagri_mappingsDBApi.findAllAutocomplete(
    req.query.query,
    req.query.limit,
  );

  res.status(200).send(payload);
});

/**
 * @swagger
 *  /api/booking_samagri_mappings/{id}:
 *    get:
 *      security:
 *        - bearerAuth: []
 *      tags: [Booking_samagri_mappings]
 *      summary: Get selected item
 *      description: Get selected item
 *      parameters:
 *        - in: path
 *          name: id
 *          description: ID of item to get
 *          required: true
 *          schema:
 *            type: string
 *      responses:
 *        200:
 *          description: Selected item successfully received
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Booking_samagri_mappings"
 *        400:
 *          description: Invalid ID supplied
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Item not found
 *        500:
 *          description: Some server error
 */

router.get(
  '/:id',
  wrapAsync(async (req, res) => {

    const payload = await Booking_samagri_mappingsDBApi.findBy({
      id: req.params.id,
    });

    res.status(200).send(payload);
  }),
);

/**
 * @swagger
 *  /api/booking_samagri_mappings/booking/{id}:
 *    get:
 *      security:
 *        - bearerAuth: []
 *      tags: [Booking_samagri_mappings]
 *      summary: Get selected item
 *      description: Get selected item
 *      parameters:
 *        - in: path
 *          name: id
 *          description: ID of item to get
 *          required: true
 *          schema:
 *            type: string
 *      responses:
 *        200:
 *          description: Selected item successfully received
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Booking_samagri_mappings"
 *        400:
 *          description: Invalid ID supplied
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Item not found
 *        500:
 *          description: Some server error
 */
router.get(
  '/booking/:id',
  wrapAsync(async (req, res) => {

    const data = await Booking_samagri_mappingsService.bookingSamagri(
      req.params.id,
      req.currentUser
    );

    res.status(200).send(data);
  }),
);

router.use('/', require('../helpers').commonErrorHandler);

module.exports = router;
