const express = require('express');

const PujaModelsDBApi = require('../db/api/puja_models');
const wrapAsync = require('../helpers').wrapAsync;

const router = express.Router();

/**
 *  @swagger
 *  /api/puja_models:
 *    post:
 *      security:
 *        - bearerAuth: []
 *      tags: [Puja_models]
 *      summary: Add new item
 *      description: Add new item
 *      requestBody:
 *        required: true
 *        content:
 *          application/json:
 *            schema:
 *              properties:
 *                data:
 *                  description: Data of the created item
 *                  type: object
 *                  $ref: "#/components/schemas/Pujas"
 *      responses:
 *        200:
 *          description: The item was successfully added
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Pujas"
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        405:
 *          description: Invalid input data
 *        500:
 *          description: Some server error
 */

router.post('/', async (req, res) => {
  const data = await PujaModelsDBApi.create(req.body.data, req.currentUser);
  // const payload = true;

  if (data.statusCode === 500) {
    res.status(500).send(data);
  } else {
    res.status(200).send(data);
  }
});

/**
 *  @swagger
 *  /api/puja_models:
 *    get:
 *      security:
 *        - bearerAuth: []
 *      tags: [Puja_models]
 *      summary: Get all puja models
 *      description: Get all puja models
 *      responses:
 *        200:
 *          description: Puja models list successfully received
 *          content:
 *            application/json:
 *              schema:
 *                type: array
 *                items:
 *                  $ref: "#/components/schemas/Pujas"
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Data not found
 *        500:
 *          description: Some server error
 */

router.get(
  '/',
  wrapAsync(async (req, res) => {
    const payload = await PujaModelsDBApi.findAll(req.query);
    res.status(200).send(payload);
  }),
);

/**
 * @swagger
 *  /api/puja_models/{id}:
 *    get:
 *      security:
 *        - bearerAuth: []
 *      tags: [Puja_models]
 *      summary: Get selected item
 *      description: Get selected item
 *      parameters:
 *        - in: path
 *          name: id
 *          description: ID of item to get
 *          required: true
 *          schema:
 *            type: string
 *      responses:
 *        200:
 *          description: Selected item successfully received
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Puja_samagri_mappings"
 *        400:
 *          description: Invalid ID supplied
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Item not found
 *        500:
 *          description: Some server error
 */

router.get(
  '/:id',
  wrapAsync(async (req, res) => {
    const payload = await PujaModelsDBApi.findBy({
      id: req.params.id,
    });
    res.status(200).send(payload);
  }),
);

/**
 *  @swagger
 *  /api/puja_models/{id}:
 *    put:
 *      security:
 *        - bearerAuth: []
 *      tags: [Puja_models]
 *      summary: Update the data of the selected item
 *      description: Update the data of the selected item
 *      parameters:
 *        - in: path
 *          name: id
 *          description: Item ID to update
 *          required: true
 *          schema:
 *            type: string
 *      requestBody:
 *        description: Set new item data
 *        required: true
 *        content:
 *          application/json:
 *            schema:
 *              properties:
 *                id:
 *                  description: ID of the updated item
 *                  type: string
 *                data:
 *                  description: Data of the updated item
 *                  type: object
 *                  $ref: "#/components/schemas/Puja_samagri_mappings"
 *              required:
 *                - id
 *      responses:
 *        200:
 *          description: The item data was successfully updated
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Puja_samagri_mappings"
 *        400:
 *          description: Invalid ID supplied
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Item not found
 *        500:
 *          description: Some server error
 */

router.put(
  '/:id',
  wrapAsync(async (req, res) => {

    await PujaModelsDBApi.update(req.body.data, req.body.id, req);
    const payload = true;
    res.status(200).send(payload);
  }),
);

/**
 * @swagger
 *  /api/puja_models/{id}:
 *    delete:
 *      security:
 *        - bearerAuth: []
 *      tags: [Puja_models]
 *      summary: Delete the selected item
 *      description: Delete the selected item
 *      parameters:
 *        - in: path
 *          name: id
 *          description: Item ID to delete
 *          required: true
 *          schema:
 *            type: string
 *      responses:
 *        200:
 *          description: The item was successfully deleted
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Puja_samagri_mappings"
 *        400:
 *          description: Invalid ID supplied
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Item not found
 *        500:
 *          description: Some server error
 */

router.delete(
  '/:id',
  wrapAsync(async (req, res) => {

    await PujaModelsDBApi.remove(req.params.id, req.currentUser);
    const payload = true;
    res.status(200).send(payload);
  }),
);

/**
 *  @swagger
 *  /api/puja_models/get_samagri_cost:
 *    post:
 *      security:
 *        - bearerAuth: []
 *      tags: [Puja_models]
 *      summary: To get the samagri cost
 *      description: To get the samagri cost
 *      requestBody:
 *        required: false
 *        content:
 *          application/json:
 *            schema:
 *              properties:
 *                data:
 *                  description: Data of the created item
 *                  type: object
 *                  $ref: "#/components/schemas/Pujas"
 *      responses:
 *        200:
 *          description: The samagri cost was fetched successfully
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Pujas"
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        405:
 *          description: Invalid input data
 *        500:
 *          description: Some server error
 */

router.post(
  '/get_samagri_cost',
  wrapAsync(async (req, res) => {
    const data = await PujaModelsDBApi.totalSamagri(req.body.data);
    res.status(200).send(data);
  }),
);

router.use('/', require('../helpers').commonErrorHandler);

module.exports = router;
