const express = require('express');

const Puja_models_samagri_mappingsDBApi = require('../db/api/puja_models_samagri_mappings');
const wrapAsync = require('../helpers').wrapAsync;

const router = express.Router();

/**
 *  @swagger
 *  /api/puja_models_samagri_mappings:
 *    post:
 *      security:
 *        - bearerAuth: []
 *      tags: [Puja_models_samagri_mappings]
 *      summary: Add new item
 *      description: Add new item
 *      requestBody:
 *        required: true
 *        content:
 *          application/json:
 *            schema:
 *              properties:
 *                data:
 *                  description: Data of the created item
 *                  type: object
 *                  $ref: "#/components/schemas/Pujas"
 *      responses:
 *        200:
 *          description: The item was successfully added
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Pujas"
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        405:
 *          description: Invalid input data
 *        500:
 *          description: Some server error
 */

router.post('/', async (req, res) => {
  await Puja_models_samagri_mappingsDBApi.create(
    req.body.data,
    req.currentUser,
  );
  const payload = true;
  res.status(200).send(payload);
});

/**
 *  @swagger
 *  /api/puja_models_samagri_mappings:
 *    get:
 *      security:
 *        - bearerAuth: []
 *      tags: [Puja_models_samagri_mappings]
 *      summary: Get all puja models
 *      description: Get all puja models
 *      responses:
 *        200:
 *          description: Puja models list successfully received
 *          content:
 *            application/json:
 *              schema:
 *                type: array
 *                items:
 *                  $ref: "#/components/schemas/Pujas"
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Data not found
 *        500:
 *          description: Some server error
 */

router.get(
  '/',
  wrapAsync(async (req, res) => {
    const payload = await Puja_models_samagri_mappingsDBApi.findAll(req.query);
    res.status(200).send(payload);
  }),
);

/**
 * @swagger
 *  /api/puja_models_samagri_mappings/{id}:
 *    get:
 *      security:
 *        - bearerAuth: []
 *      tags: [Puja_models_samagri_mappings]
 *      summary: Get selected item
 *      description: Get selected item
 *      parameters:
 *        - in: path
 *          name: id
 *          description: ID of item to get
 *          required: true
 *          schema:
 *            type: string
 *      responses:
 *        200:
 *          description: Selected item successfully received
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Puja_samagri_mappings"
 *        400:
 *          description: Invalid ID supplied
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Item not found
 *        500:
 *          description: Some server error
 */

router.get(
  '/:id',
  wrapAsync(async (req, res) => {

    const payload = await Puja_models_samagri_mappingsDBApi.findBy({
      id: req.params.id,
    });

    res.status(200).send(payload);
  }),
);

/**
 *  @swagger
 *  /api/puja_models_samagri_mappings:
 *    put:
 *      security:
 *        - bearerAuth: []
 *      tags: [Puja_models_samagri_mappings]
 *      summary: Update the data of the selected item
 *      description: Update the data of the selected item
 *      parameters:
 *          schema:
 *            type: string
 *      requestBody:
 *        description: Set new item data
 *        required: true
 *        content:
 *          application/json:
 *            schema:
 *              properties:
 *                data:
 *                  description: Data of the updated item
 *                  type: object
 *                  $ref: "#/components/schemas/Puja_samagri_mappings"
 *              required:
 *                - id
 *      responses:
 *        200:
 *          description: The item data was successfully updated
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Puja_samagri_mappings"
 *        400:
 *          description: Invalid ID supplied
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Item not found
 *        500:
 *          description: Some server error
 */

router.put(
  '/',
  wrapAsync(async (req, res) => {
    await Puja_models_samagri_mappingsDBApi.update(req.body.data, req);
    const payload = true;
    res.status(200).send(payload);
  }),
);

/**
 * @swagger
 *  /api/puja_models_samagri_mappings/delete:
 *    post:
 *      security:
 *        - bearerAuth: []
 *      tags: [Puja_models_samagri_mappings]
 *      summary: Delete the selected item
 *      description: Delete the selected item
 *      parameters:
 *          schema:
 *            type: string
 *      requestBody:
 *        description: Set new item data
 *        required: true
 *        content:
 *          application/json:
 *            schema:
 *              properties:
 *                data:
 *                  description: Data of the updated item
 *                  type: object
 *                  $ref: "#/components/schemas/Puja_samagri_mappings"
 *              required:
 *                - id
 *      responses:
 *        200:
 *          description: The item was successfully deleted
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Puja_samagri_mappings"
 *        400:
 *          description: Invalid ID supplied
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Item not found
 *        500:
 *          description: Some server error
 */

router.post(
  '/delete',
  wrapAsync(async (req, res) => {
    await Puja_models_samagri_mappingsDBApi.remove(
      req.body.data,
      req.currentUser,
    );
    const payload = true;
    res.status(200).send(payload);
  }),
);

router.use('/', require('../helpers').commonErrorHandler);

module.exports = router;
