const express = require('express');

const UsersService = require('../services/users');
const UsersDBApi = require('../db/api/users');
const wrapAsync = require('../helpers').wrapAsync;

const router = express.Router();

/**
 *  @swagger
 *  components:
 *    schemas:
 *      Users:
 *        type: object
 *        properties:

 *          firstName:
 *            type: string
 *            default: firstName
 *          lastName:
 *            type: string
 *            default: lastName
 *          phoneNumber:
 *            type: string
 *            default: phoneNumber
 *          email:
 *            type: string
 *            default: email
 *          forgot_password_token:
 *            type: string
 *            default: forgot_password_token
 *          remember_token:
 *            type: string
 *            default: remember_token

 *          first_time_login:
 *            type: integer
 *            format: int64
 *          is_active:
 *            type: integer
 *            format: int64

 *          
 *          role:
 *            type: string
 *            default: user
 *          
 *          type:
 *            type: string
 *            default: user
 */

/**
 *  @swagger
 * tags:
 *   name: Users
 *   description: The Users managing API
 */

/**
 *  @swagger
 *  /api/users:
 *    post:
 *      security:
 *        - bearerAuth: []
 *      tags: [Users]
 *      summary: Add new item
 *      description: Add new item
 *      requestBody:
 *        required: true
 *        content:
 *          application/json:
 *            schema:
 *              properties:
 *                data:
 *                  description: Data of the updated item
 *                  type: object
 *                  $ref: "#/components/schemas/Users"
 *      responses:
 *        200:
 *          description: The item was successfully added
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Users"
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        405:
 *          description: Invalid input data
 *        500:
 *          description: Some server error
 */

router.post('/', async (req, res) => {
  try {
    console.log('req', req);
    await UsersService.create(
      req.body.data,
      req.currentUser,
      true,
      req.headers.referer,
    );
    const payload = true;
    console.log('payload', payload);
    res.status(200).send(payload);
  } catch (err) {
    console.log(err);
  }
});

/**
 *  @swagger
 *  /api/users/{id}:
 *    put:
 *      security:
 *        - bearerAuth: []
 *      tags: [Users]
 *      summary: Update the data of the selected item
 *      description: Update the data of the selected item
 *      parameters:
 *        - in: path
 *          name: id
 *          description: Item ID to update
 *          required: true
 *          schema:
 *            type: string
 *      requestBody:
 *        description: Set new item data
 *        required: true
 *        content:
 *          application/json:
 *            schema:
 *              properties:
 *                id:
 *                  description: ID of the updated item
 *                  type: string
 *                data:
 *                  description: Data of the updated item
 *                  type: object
 *                  $ref: "#/components/schemas/Users"
 *              required:
 *                - id
 *      responses:
 *        200:
 *          description: The item data was successfully updated
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Users"
 *        400:
 *          description: Invalid ID supplied
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Item not found
 *        500:
 *          description: Some server error
 */

router.put(
  '/:id',
  wrapAsync(async (req, res) => {
    try {
      console.log('req', req);
      await UsersService.update(req.body.data, req.body.id, req.currentUser);
      const payload = true;
      console.log('payload', payload);
      res.status(200).send(payload);
    } catch (err) {
      console.log(err);
    }
  }),
);

/**
 * @swagger
 *  /api/users/{id}:
 *    delete:
 *      security:
 *        - bearerAuth: []
 *      tags: [Users]
 *      summary: Delete the selected item
 *      description: Delete the selected item
 *      parameters:
 *        - in: path
 *          name: id
 *          description: Item ID to delete
 *          required: true
 *          schema:
 *            type: string
 *      responses:
 *        200:
 *          description: The item was successfully deleted
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Users"
 *        400:
 *          description: Invalid ID supplied
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Item not found
 *        500:
 *          description: Some server error
 */

router.delete(
  '/:id',
  wrapAsync(async (req, res) => {
    try {
      console.log('req.params.id', req.params.id);
      console.log('req.currentUser', req.currentUser);
      await UsersService.remove(req.params.id, req.currentUser);
      const payload = true;
      console.log('payload', payload);
      res.status(200).send(payload);
    } catch (err) {
      console.log(err);
    }
  }),
);

/**
 *  @swagger
 *  /api/users:
 *    get:
 *      security:
 *        - bearerAuth: []
 *      tags: [Users]
 *      summary: Get all users
 *      description: Get all users
 *      responses:
 *        200:
 *          description: Users list successfully received
 *          content:
 *            application/json:
 *              schema:
 *                type: array
 *                items:
 *                  $ref: "#/components/schemas/Users"
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Data not found
 *        500:
 *          description: Some server error
 */

router.get(
  '/',
  wrapAsync(async (req, res) => {
    try {
      console.log('req.query', req.query);
      const payload = await UsersDBApi.findAll(req.query);
      console.log('payload', payload);
      res.status(200).send(payload);
    } catch (err) {
      console.log(err);
    }
  }),
);

router.get('/autocomplete', async (req, res) => {
  try {
    console.log(' req.query', req.query);
    const payload = await UsersDBApi.findAllAutocomplete(
      req.query.query,
      req.query.limit,
    );
    console.log('payload', payload);
    res.status(200).send(payload);
  } catch (err) {
    console.log(err);
  }
});

/**
 * @swagger
 *  /api/users/{id}:
 *    get:
 *      security:
 *        - bearerAuth: []
 *      tags: [Users]
 *      summary: Get selected item
 *      description: Get selected item
 *      parameters:
 *        - in: path
 *          name: id
 *          description: ID of item to get
 *          required: true
 *          schema:
 *            type: string
 *      responses:
 *        200:
 *          description: Selected item successfully received
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Users"
 *        400:
 *          description: Invalid ID supplied
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Item not found
 *        500:
 *          description: Some server error
 */

router.get(
  '/:id',
  wrapAsync(async (req, res) => {
    try {
      console.log('req.params.id ', req.params.id);
      const payload = await UsersDBApi.findBy({ id: req.params.id });

      delete payload.password;
      console.log('payload', payload);
      res.status(200).send(payload);
    } catch (err) {
      console.log(err);
    }
  }),
);

router.use('/', require('../helpers').commonErrorHandler);

module.exports = router;
